Introduction
============

This document describes the interface between britney and autopkgtest running in
Jenkins.

General Workflow
================

 1. **britney** asks **adt-britney** (command 'request') which package want tests for a new package version
 2. **adt-britney** generates a request file with the list of packges that will be
 tested and the dependencies that are causing the build.
 3. **britney** asks **adt-britney** (command 'collect') for the list of results since last run and updates
 its list of pass/fail
 4. **britney** generate excuses
     * britney substracts known-uninstallable rdepends from requests file
     * any package/version(/rdepends) tuple which is due for testing but does not yet have a pass/fail result is considered not a valid candidate
 5. **britney** generates an updated request file with remaining rdepends and
 submits it to **adt-britney** (command 'submit')
 6. **adt-britney** generate a state file per package with test and sync it to the
 QA lab
 7. In the QA Lab, **adt-jenkins** processes theses state files and wait until it
 sees a -proposed suite in their archive with a recent enough Release timestamp.
 8. **adt-jenkins** calls *jenkins* jobs named: <release>-adt-<package>
 9. **Jenkins** starts **autopkgtest** runs in VMs and dispatch them amongst the
 available execution nodes.
 10. At the end of a build (i.e when a package/arch run ends) the result of the
 test and list of dependencies tested with their version is synced back to the
 server running adt-jenkins
 

Implementation
==============

adt-britney
-----------

This script is the interface between britney and the QA Lab. 

It provides the following commands:
 * request: Generates a request file.
 * submit: Submits a request file
 * collect: Collects test results

Check output of `adt-britney -h` for general syntax

### Configuration ###

By default, adt-britney reads its configuration from a configuration file named
**adt-britney.rc** located in the same directory than the executable. This file
can be overriden from the command line with the option **-c, --config
<filename>**.

The following parameters can be configured:

  * `arch`: Architecture to use for archive index
  * `release`: Release 
  * `aptroot`: Location of the aptroot directory to store apt package indexes
  and adt state files
  * `apturi`: URI used in sources.list e.g *http://archive.ubuntu.com* or
*file:~/Ubuntu/aptroot/mirror/ubuntu*
  * `components`: Components to enable
  * `extrasources`: Extra sources to add to sources.list and index scan
  * `rsync_host`: Remote host to running adt-jenkins

### Command request ###
The request commands generates a request file and can be called with or without
a dependency list as input.

Called without a dependency list, it will generate a request file containing the
list of packages with DEP8 tests that are newer or with newer dependencies
since last time test have been submitted.

Called with a dependency list (, it will generate a request file containing a
list of packages with DEP8 tests, which depends on a package listed in the
dependency list, which one is newer than last time test have been submitted.

The request file is written by default to stdout and can be written to a file
with the option `-O FILENAME`.

#### Dependency list format ####
The dependency list is a text file containing a sequence of lines of the
following form:

    <src package> <version>

ex:

    python-defaults 3.4.5
    apt 1.2.3


#### request file format ####
Sequence of text lines of the following form:

    <header>
    <package list>

    <header>: Suite name and timestamp of the archive

    Suite: code name
    Date: <timestamp>

    <package list>: List of <package with test>

    <package with test>: package, status and dependencies
    <src package> <version> NEW [<src package> <version> ...]

ex:

    Suite: raring
    Date: Thu, 22 Nov 2012  7:48:28 UTC
    Suite: raring-proposed
    Date: Thu, 22 Nov 2012  8:36:08 UTC
    unity-firefox-extension 2.4.1-0ubuntu2 NEW firefox 17.0~b6+build1-0ubuntu1
    upower 0.9.17-1build1 NEW glib2.0 2.34.2-1 upower 0.9.17-1build1

To generate a request file with `-proposed` enabled from a list of dependency
`depends.list` and write it to `packages.request` run:
  
    $ ./adt-britney -P request -O packages.request depends.list


### Command submit ###
This command takes a request file as argument, splits it into one state file per
package to test and send these state files to the host that will trigger the
tests.

A state file is a json file and contains the following information:
  * source package and version
  * status of the test on each architecture
  * list of all the binary dependencies and their version of the binaries
  generated from the source package.
  * list of all the source package and their version that cause the test.

This file will be updated with the results of the tests per architecture to know
the overall status of the test for the package.

To submit a request file called `packages.request` run the command:

    $ ./adt-britney submit packages.request


state files are stored in `$APTROOT/$RELEASE(-proposed)/$ARCH/var/cache/adt`.

state files are rsynced to the remote host to `$RSYNC_HOST/out/`

### Command collect ###
This command syncs results files from the host that triggered the tests and
calculates the status for a package once all the tests ran on all the
architectures. The it updates the state files, archive the state file which
global status is 'PASS' or 'FAIL', archives the result files that have been
collected and outputs the new results since last collect in alphabetical and
chronological order of the tests.

To collect the results run the command:

    $ ./adt-britney collect -O results.output

Results are stored in the same directory than the state files.

#### result file format ####
Sequence of text lines of the following form:

    <src package> <version> <status> [<src package> <version> ...]


Status in a result file can be `PASS` or `FAIL`

adt-jenkins
-----------
This script scan the incoming directory for new state files and triggers package
test when the version of the package in the archive greater or equal to the
version in the request file.

It use the same configuration file than adt-britney and a jenkins credentials
file to 

### Configuration ###
This script requires 2 configuration files:

 * adt-britney.rc: General configuration (see adt-britney above)
 * adt.credentials: Jenkins credentials file passed on the command line with the
 * option -J. The content of the file is:

        jenkins:
          username: jenkins username with create/modify/build job privileges
          password: user password
          url: URL of master jenkins
          token: token to trigger remote jobs


### Submitting a job ###
To submit a job run the following command:

        $ /adt-jenkins -P -c ~/adt-britney.rc -J ~/.adt.credentials --submit $statefile

**$statefile** is a path to a state file

This command will create a job with the name ``RELEASE-adt-PACKAGE`` or update
it if it already exists and start the job remotely.
