Introduction
============

This document describes the steps to follow at the beginning of the cycle to open a new release

Testing environment
===================

The first step is to create the base testing environment. As cloud images are
not yet created so early in the cycle, base images are created by copying
pristine images of the latest stable release to the devel release and upgrading
them manually.

This part of the configuration must be done on every test nodes.
Images are kept in a cache directory (usually $HOME/cache/disks)

 * Copy the pristine image and its seed

   $ cd $HOME/cache/disks/
   $ cp pristine-saucy-i386-20131020_080554.img pristine-trusty-i386-20131020_080554.img
   $ cp pristine-saucy-i386-20131020_080554.img.seed pristine-trusty-i386-20131020_080554.img.seed
   $ ln -sf $(pwd)/pristine-trusty-i386-20131020_080554.img $(pwd)/pristine-trusty-i386.img

 * Start the base image

   $ IMGID=i386-20131020_080554
   $ /usr/bin/qemu-system-x86_64 -enable-kvm -m 3072 -smp 2 -monitor stdio -localtime -no-reboot -net user -redir tcp:54399::22 -drive file=pristine-trusty-${IMGID}.img,if=virtio -drive file=pristine-trusty-${IMGID}.img.seed,if=virtio -vnc localhost:99 -pidfile pristine-trusty-${IMGID}.img.pid -readconfig $HOME/auto-package-testing//etc/vm.cfg

 * Update sources.list and upgrade

   $ ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o CheckHostIP=no -i /var/cache/adt/disks/adtkey -tt -p 54399 -l ubuntu localhost
   $ find /etc/apt -name "*.list" | sudo xargs sed -i "s/saucy/trusty/"
   $ sudo apt-get update && sudo apt-get -y dist-upgrade

 * Verify that it works with a small package. Check that the development release
is used and proposed is enabled.

   $ ~/auto-package-testing/bin/run-adt-test -r $(distro-info -d) -a amd64 -Pds tabix

Britney
=======

On britney host, after initialization of the new directory structure for the
new series, carry over history from previous series to detect regressions
between series.

Copy
$HOME/proposed-migration/autopkgtest/data/adt/<RELEASE-1>-proposed/amd64/results.history
to
$HOME/proposed-migration/autopkgtest/data/adt/<RELEASE>-proposed/amd64/

where <RELEASE-1> is the code name of previous release, and RELEASE is the
code name of the current development release.

Jenkins
=======

This part of the configuration must be done on the server that host jenkins
server.

 * in /var/lib/jenkins/QA/run-adt-jenkins add the new development release to
RELEASES. Ideally it should be $(distro-info --supported) but support for
autopkgtest started in precise.

 * This job runs every 10min, monitor run-jenkins.${CODENAME}.log for any
failure. More paticulary verify that the file
/var/lib/jenkins/QA/adt/aptroot/${CODENAME}-proposed/amd64/etc/apt/sources.list
has been created correctly and contains entries for all pockets and components.

 * Jobs will be triggers when files appear in /var/local/adt/trusty/in/. When
cron job start, verify that jobs are created and started on jenkins web site.

 * On jenkins create the views 
   * <DEV RELEASE>
   * <DEV RELEASE>/AutoPkgTest

 * The view <DEV RELEASE>/AutoPkgTest is a dashboard view with the following
configuration:
   * Include jobs matching the regular expression: trusty(-proposed|-ppa)?-(adt|pkg).*(?<!armhf)$
   * Left column: "Unstable Jobs" and "Job Statistics"
   * Right column: "Latest builds" with 25 builds
   * Bottom of the page: "Test Statistics Grid"

 * The same views must be created on private and public instances.

 * Create the job <RELEASE>-adt-setup-testbed by copying it from the previous
release and update the name of the release in the trigger URL and the command of
the first execution step.
